<?php
require_once dirname(__FILE__).'/PriceByRoleTestCase.php';

class WooProductsSkuManagerTest extends PriceByRoleTestCase {
       
    public $skuManager;
    
    public function setUp()
    {
        parent::setUp();
        
        $file = 'WooProductsSkuManager.php';
        require_once $this->getPluginPath('common/import/'.$file);
        
        $this->doInitSkuManager();
        $this->doCreateProduct();
    }
    
    public function doInitSkuManager()
    {
        $engine = 'testEngine';
        $config = 'testConfig';
        $language = 'testDomain';
        
        $this->skuManager = new WooProductsSkuManager(
            $config,
            $engine,
            $language
        ); 
    }
    
    public function getTestingPost($id) 
    {
        return get_post($id);
    }
    
    public function doCreateProduct()
    {
        parent::doCreateProduct();
        
        $id = $this->getProductId('simple');
        update_post_meta($id, '_stock_status', 'outofstock');
        
        $this->skuManager->exsistProduct = $this->getTestingPost($id);
    }
    
    public function getTestCases()
    {
        $cases = array(
        // _stock_status option exists in imported file
            array(
                'option'   => '_stock_status',
                'value'    => 'instock',
                'expected' => 'instock'
            ), 
        // _stock option exists in imported file
            array(
                'option'   => '_stock',
                'value'    => '10',
                'expected' => 'instock'
            ),
        // no options in imported file, get option from exsiting post
            array(
                'option'   => '',
                'expected' => 'outofstock'
            )
        );
        
        return $cases;
    }

    public function isOptionExsits($value)
    {
        return !empty($value);
    }

    public function setCaseOptions($case)
    {
        $this->doResetOptions();
        
        if ($this->isOptionExsits($case['option'])) {
            $key = $case['option'];
            $value = $case['value'];    
                
            $this->skuManager->newPostMeta[$key] = $value;
        }
    }
    
    public function doResetOptions()
    {
        $options = array(
            '_stock_status',
            '_stock'
        );
        
        foreach ($options as $option) {
            unset($this->skuManager->newPostMeta[$option]);
        }
    }
    
    public function testSetStockStatusForProduct()
    {
        $cases = $this->getTestCases();
        
        foreach ($cases as $case) {
            $this->setCaseOptions($case);
            
            $testedValue = $this->skuManager->setStockStatusForProduct();
            
            $this->assertEquals($case['expected'], $testedValue);
        } 
    }
    
    
}
