<?php
require_once dirname(__FILE__).'/PriceByRoleTestCase.php';

class WooUserRolePricesBackendFestiPluginTest extends PriceByRoleTestCase
{
    private $_updateProductOptionsHookName = 'woocommerce_product_options_pricing';
    private $_listenerMethodName = 'onAppendFildsToSimpleOptionsAction';
    private $_listenerPriority = 10;
    private $_settingsOptionsKey = 'settings';
    
    public function setUp()
    {  
        parent::setUp();
        
        require_once $this->getPluginPath('WooUserRolePricesFestiPlugin.php');
        
        $file = 'WooUserRolePricesBackendFestiPlugin.php';
        require_once $this->getPluginPath($file);

        $this->_backend = new WooUserRolePricesBackendFestiPlugin(
            $this->pluginMainFile
        );
        $this->_backend->onInstall();
        $this->doClearCache();
        $this->doCreateProduct();
    } // end setUp
    
    public function testDuplicatingPricesFields()
    {
        $this->updateOptions(
            PRICE_BY_ROLE_OPTIONS_PREFIX.$this->_settingsOptionsKey,
            array(
                'roles' => array(
                    'administrator' => true
                )
            )
        );
        
        $path = $this->getWooCommercePath(
            'includes/admin/wc-meta-box-functions.php'
        );
        
        require_once $path;

        add_filter(
            $this->_updateProductOptionsHookName,
            array($this->_backend, $this->_listenerMethodName),
            $this->_listenerPriority
        );
        
        $productId = $this->getProductId('simple');

        $GLOBALS['post'] = get_post($productId);
        
        $listenersList = $this->_getListenersOfWoocommerceAction();
        ob_start();
        $this->_backend->onAppendFildsToSimpleOptionsAction();
        ob_get_clean();

        $result = $this->_hasPluginListenerOfPrintPriceFieldAction();
        
        $this->assertTrue($result);
    } // end testDuplicatingPricesFields
    
    private function _getListenersOfWoocommerceAction()
    {
        $hookName = $this->_updateProductOptionsHookName;
        $listenersList =  $GLOBALS['wp_filter'][$hookName];

        return $listenersList;
    } // end _getListenersOfWoocommerceAction
    
    private function _hasPluginListenerOfPrintPriceFieldAction()
    {
        $listenersList = $this->_getListenersOfWoocommerceAction();

        return $listenersList == false;
    } // end _hasPluginListenerOfPrintPriceFieldAction    
    
    public function testEmptyRolePricesProductMetaOptionsInUpdateAction()
    {
        $productId = $this->getProductId('simple');
        $this->_backend->onUpdateAllTypeProductMetaOptionsAction($productId);
        
        $rolePriceList = get_post_meta (
            $productId,
            PRICE_BY_ROLE_PRICE_META_KEY,
            true
        );
        
        $rolePriceList = json_decode($rolePriceList, true);
        
        $result = is_array($rolePriceList) && empty($rolePriceList);
        
        $this->assertTrue($result);
    } // end testEmptyRolePricesProductMetaOptionsInUpdateAction
}