<?php
require_once dirname(__FILE__).'/PriceByRoleTestCase.php';

class WooUserRolePricesFrontendFestiPluginTest extends PriceByRoleTestCase 
{
    public $idPost;
    public $cart;
    
    public function setUp()
    {
        parent::setUp();
        
        require_once $this->getPluginPath('WooUserRolePricesFestiPlugin.php');
        
        $file = 'WooUserRolePricesFrontendFestiPlugin.php';
        require_once $this->getPluginPath($file);
        $file = 'common/woocommerce/product/FestiWooCommerceProduct.php';
        require_once $this->getPluginPath($file);
                
        $this->cart = $this->getCart();
        
        $this->_frontend = new WooUserRolePricesFrontendFestiPlugin(
            $this->pluginMainFile
        );
        
        $this->setFrontendPropertyValue();  
        
        $this->doCreateProduct();
        
        $this->doCartFill();
        
    }
    
    public function getCart()
    {
        $cartController = WooCommerceCartFacade::getInstance();
        return $cartController->getCartInstance();
    }

    public function setFrontendPropertyValue()
    {
        $productController = new FestiWooCommerceProduct($this->_frontend);
        $reflectionClass = new ReflectionClass(
            'WooUserRolePricesFrontendFestiPlugin'
        );
        $reflectionProperty = $reflectionClass->getProperty('products');
        $reflectionProperty->setAccessible(true);
        
        $reflectionProperty->setValue($this->_frontend, $productController);
    }
    
    public function doCreateProduct() 
    {
        parent::doCreateProduct();
        $this->idPost = $this->getProductId('simple');
        
        update_post_meta($this->idPost, '_regular_price', '100');
        update_post_meta($this->idPost, '_price', '100');
    }
    
    public function doCartFill()
    {
        $this->cart->add_to_cart($this->idPost, 1);
    }
    
    public function setCartProperties($options) 
    {
        $this->cart->subtotal_ex_tax = $options['subtotal_ex_tax'];
        $this->cart->tax_total = $options['tax_total'];
        $this->cart->prices_include_tax = $options['prices_include_tax'];
    }

    public function getCartCasesProperties()
    {
        $properties = array(
        //tax included in price, displays inclusive
            array(
                'subtotal_ex_tax'    => 80,
                'tax_total'          => 20,
                'tax_option_display' => 'incl',
                'prices_include_tax' => 1,
                'expected'           => '100'
            ),
        //tax included in price, displays separately
            array(
                'subtotal_ex_tax'    => 80,
                'tax_total'          => 20,
                'tax_option_display' => 'excl',
                'prices_include_tax' => 1,
                'expected'           => '100'
            ),
        //tax excluded from price, displays inclusive
            array(
                'subtotal_ex_tax'    => 100,
                'tax_total'          => 25,
                'tax_option_display' => 'incl',
                'prices_include_tax' => '',
                'expected'           => '125'
            ),
        //tax excluded from price, displays separately
            array(
                'subtotal_ex_tax'    => 100,
                'tax_total'          => 25,
                'tax_option_display' => 'excl',
                'prices_include_tax' => '',
                'expected'           => '125'
            ),
        );
        
        return $properties;
    }

    public function getReflectedFunction()
    {
        $testedFunction = new ReflectionMethod(
            'WooUserRolePricesFrontendFestiPlugin',
            'getRetailSubTotalWithTax'
        );
        
        $testedFunction->setAccessible(true);
        
        return $testedFunction;
    }

    public function testGetRetailSubTotalWithTax()
    {
        $cases = $this->getCartCasesProperties();
        
        $testedFunction = $this->getReflectedFunction();
         
        foreach ($cases as $case) {
            $this->setCartProperties($case);
            
            $currentTotal = $testedFunction->invoke($this->_frontend);

            $this->assertEquals($case['expected'], $currentTotal);
        }
    }
    
}
